==> spirit_hub-1.0.0.5.tar.gz: spirit_hub-1.0.0.5/app/assets/javascripts/spirit_hub/controllers/pdf_annotation_controller.js <==
import { Controller } from '@hotwired/stimulus';

const DRAG_HANDLE_SIZE = 16;
export default class extends Controller {
  static targets = ['input', 'deleteButton'];

  static values = {
    pageIndex: Number,
    id: String,
    fontSize: { type: Number, default: 14 },
    color: { type: Array, default: [0, 0, 0] },
    text: String,
    canvasId: String,
    x: Number,
    y: Number,
  };

  connect() {
    this.setupElement();
    this.setupEventHandlers();

    if (this.hasTextValue && this.textValue) {
      this.inputTarget.innerText = this.textValue;
    }

    if (this.hasTextValue && !this.textValue) {
      setTimeout(() => {
        this.inputTarget.contentEditable = true;
        this.inputTarget.focus();
      }, 0);
    }
  }

  get canvasElement() {
    return document.getElementById(this.canvasIdValue);
  }

  setupElement() {
    if (this.hasXValue && this.hasYValue) {
      this.element.style.left = `${this.xValue}px`;
      this.element.style.top = `${this.yValue}px`;
    }

    this.element.style.position = 'absolute';
    this.element.style.minWidth = '100px';
    this.element.style.minHeight = '20px';
    this.element.style.background = 'rgba(255, 255, 255, 0.8)';
    this.element.style.border = '1px solid #000';
    this.element.style.borderLeft = `${DRAG_HANDLE_SIZE}px solid #000`;
    this.element.style.padding = '4px';
    this.element.style.paddingLeft = '8px';
    this.element.style.fontSize = `${this.fontSizeValue}px`;
    this.element.style.zIndex = '20';

    if (this.hasInputTarget) {
      this.inputTarget.style.outline = 'none';
      this.inputTarget.style.cursor = 'text';
      this.inputTarget.style.width = '100%';
      this.inputTarget.style.height = '100%';

      if (this.hasColorValue) {
        this.inputTarget.style.color = this.rgbToHex(this.colorValue);
      }
    }

    if (this.hasDeleteButtonTarget) {
      this.deleteButtonTarget.style.position = 'absolute';
      this.deleteButtonTarget.style.bottom = '-20px';
      this.deleteButtonTarget.style.right = '40%';
      this.deleteButtonTarget.style.width = '20px';
      this.deleteButtonTarget.style.height = '20px';
      this.deleteButtonTarget.style.background = 'rgba(118, 118, 118, 0.74)';
      this.deleteButtonTarget.style.color = 'white';
      this.deleteButtonTarget.style.borderRadius = '50%';
      this.deleteButtonTarget.style.display = 'flex';
      this.deleteButtonTarget.style.alignItems = 'center';
      this.deleteButtonTarget.style.justifyContent = 'center';
      this.deleteButtonTarget.style.fontSize = '12px';
      this.deleteButtonTarget.style.cursor = 'pointer';
      this.deleteButtonTarget.style.visibility = 'hidden';
      this.deleteButtonTarget.style.zIndex = '1001';
    }
  }

  setupEventHandlers() {
    this.element.dataset.action = `
      mousemove->pdf-annotation#handleMouseMove
      mouseenter->pdf-annotation#showDeleteButton
      mouseleave->pdf-annotation#hideDeleteButton
      mousedown->pdf-annotation#startDragging
    `;

    this.inputTarget.dataset.action = `
      dblclick->pdf-annotation#makeEditable
      blur->pdf-annotation#saveEditedText
      keydown->pdf-annotation#handleKeyDown
    `;

    this.deleteButtonTarget.dataset.action = 'click->pdf-annotation#delete';
  }

  makeEditable(event) {
    const innerText = event.target;
    innerText.contentEditable = true;
    innerText.focus();

    const selection = window.getSelection();
    const range = document.createRange();
    range.selectNodeContents(innerText);
    selection.removeAllRanges();
    selection.addRange(range);
  }

  saveEditedText(event) {
    const target = event.target;
    target.contentEditable = false;
    const text = target.innerText.trim();

    if (text !== '') {
      this.textValue = text;

      // Dispatch the updated event to the window so pdf-viewer can catch it
      this.dispatch('updated', {
        detail: {
          id: this.idValue,
          value: this.textValue,
          rect: this.getCurrentRect(),
          fontSize: this.fontSizeValue,
          color: this.colorValue,
          pageIndex: this.pageIndexValue,
        },
      });
    } else {
      this.delete();
    }
  }

  handleKeyDown(event) {
    if (event.key === 'Enter' && !event.shiftKey) {
      event.preventDefault();
      event.target.blur();
    }
  }

  delete(event) {
    if (event) event.stopPropagation();

    this.dispatch('deleted', {
      detail: { id: this.idValue },
    });

    this.element.remove();
  }

  showDeleteButton() {
    this.deleteButtonTarget.style.visibility = 'visible';
  }

  hideDeleteButton() {
    this.deleteButtonTarget.style.visibility = 'hidden';
  }

  handleMouseMove(event) {
    const rect = this.element.getBoundingClientRect();
    const isInHandleArea = event.clientX - rect.left < DRAG_HANDLE_SIZE;
    this.element.style.cursor = isInHandleArea ? 'move' : 'default';
  }

  startDragging(event) {
    const rect = this.element.getBoundingClientRect();
    const isInHandleArea = event.clientX - rect.left < DRAG_HANDLE_SIZE;

    if (isInHandleArea) {
      this.dragOffsetX = event.clientX - rect.left;
      this.dragOffsetY = event.clientY - rect.top;

      document.addEventListener('mousemove', (this.boundDragMove = this.dragMove.bind(this)));
      document.addEventListener('mouseup', (this.boundDragEnd = this.dragEnd.bind(this)));

      document.body.style.userSelect = 'none';
      event.preventDefault();
    }
  }

  dragMove(event) {
    const canvas = this.canvasElement;
    if (!canvas) return;

    const containerRect = canvas.getBoundingClientRect();
    const newX = event.clientX - containerRect.left - this.dragOffsetX;
    const newY = event.clientY - containerRect.top - this.dragOffsetY;

    this.element.style.left = `${newX}px`;
    this.element.style.top = `${newY}px`;
  }

  dragEnd() {
    this.dispatch('moved', {
      detail: {
        id: this.idValue,
        rect: this.getCurrentRect(),
      },
    });

    document.removeEventListener('mousemove', this.boundDragMove);
    document.removeEventListener('mouseup', this.boundDragEnd);
    document.body.style.userSelect = '';
  }

  getCurrentRect() {
    const canvas = this.canvasElement;
    if (!canvas) return [0, 0, 0, 0];

    const rect = this.element.getBoundingClientRect();
    const containerRect = canvas.getBoundingClientRect();
    const x = rect.left - containerRect.left;
    const y = rect.top - containerRect.top;

    return [x, y, x + rect.width, y + rect.height];
  }

  rgbToHex(rgb) {
    return `rgb(${rgb[0]}, ${rgb[1]}, ${rgb[2]})`;
  }
}

==> spirit_hub-1.0.0.9.tar.gz: spirit_hub-1.0.0.9/app/assets/javascripts/spirit_hub/controllers/pdf_annotation_controller.js <==
import { Controller } from '@hotwired/stimulus';

const DRAG_HANDLE_SIZE = 32;
export default class extends Controller {
  static targets = ['input', 'deleteButton'];

  static values = {
    pageIndex: Number,
    id: String,
    fontSize: { type: Number, default: 14 },
    color: { type: Array, default: [0, 0, 0] },
    text: String,
    canvasId: String,
    x: Number,
    y: Number,
  };

  connect() {
    this.setupElement();
    this.setupEventHandlers();

    if (this.hasTextValue && this.textValue) {
      this.inputTarget.innerText = this.textValue;
    }

    if (this.hasTextValue && !this.textValue) {
      setTimeout(() => {
        this.inputTarget.contentEditable = true;
        this.inputTarget.focus();
      }, 0);
    }
  }

  get canvasElement() {
    return document.getElementById(this.canvasIdValue);
  }

  setupElement() {
    if (this.hasXValue && this.hasYValue) {
      this.element.style.left = `${this.xValue}px`;
      this.element.style.top = `${this.yValue}px`;
    }

    this.element.style.position = 'absolute';
    this.element.style.minWidth = '100px';
    this.element.style.minHeight = '20px';
    this.element.style.background = 'rgba(255, 255, 255, 0.8)';
    this.element.style.border = '1px solid #000';
    this.element.style.borderLeft = `${DRAG_HANDLE_SIZE}px solid #000`;
    this.element.style.padding = '4px';
    this.element.style.paddingLeft = '8px';
    this.element.style.fontSize = `${this.fontSizeValue}px`;
    this.element.style.zIndex = '20';

    if (this.hasInputTarget) {
      this.inputTarget.style.outline = 'none';
      this.inputTarget.style.cursor = 'text';
      this.inputTarget.style.width = '100%';
      this.inputTarget.style.height = '100%';

      if (this.hasColorValue) {
        this.inputTarget.style.color = this.rgbToHex(this.colorValue);
      }
    }

    if (this.hasDeleteButtonTarget) {
      this.deleteButtonTarget.style.position = 'absolute';
      this.deleteButtonTarget.style.top = '-8px';
      this.deleteButtonTarget.style.right = '-8px';
      this.deleteButtonTarget.style.width = '20px';
      this.deleteButtonTarget.style.height = '20px';
      this.deleteButtonTarget.style.background = 'rgba(118, 118, 118, 0.74)';
      this.deleteButtonTarget.style.color = 'white';
      this.deleteButtonTarget.style.borderRadius = '50%';
      this.deleteButtonTarget.style.display = 'flex';
      this.deleteButtonTarget.style.alignItems = 'center';
      this.deleteButtonTarget.style.justifyContent = 'center';
      this.deleteButtonTarget.style.fontSize = '12px';
      this.deleteButtonTarget.style.cursor = 'pointer';
      this.deleteButtonTarget.style.visibility = 'hidden';
      this.deleteButtonTarget.style.zIndex = '1001';
    }
  }

  setupEventHandlers() {
    this.element.dataset.action = `
      mousemove->pdf-annotation#handleMouseMove
      mouseenter->pdf-annotation#showDeleteButton
      mouseleave->pdf-annotation#hideDeleteButton
      mousedown->pdf-annotation#startDragging
    `;

    this.inputTarget.dataset.action = `
      dblclick->pdf-annotation#makeEditable
      blur->pdf-annotation#saveEditedText
      keydown->pdf-annotation#handleKeyDown
    `;

    this.deleteButtonTarget.dataset.action = 'click->pdf-annotation#delete';
  }

  makeEditable(event) {
    const innerText = event.target;
    innerText.contentEditable = true;
    innerText.focus();

    const selection = window.getSelection();
    const range = document.createRange();
    range.selectNodeContents(innerText);
    selection.removeAllRanges();
    selection.addRange(range);
  }

  saveEditedText(event) {
    const target = event.target;
    target.contentEditable = false;
    const text = target.innerText.trim();

    if (text !== '') {
      this.textValue = text;

      // Dispatch the updated event to the window so pdf-viewer can catch it
      this.dispatch('updated', {
        detail: {
          id: this.idValue,
          value: this.textValue,
          rect: this.getCurrentRect(),
          fontSize: this.fontSizeValue,
          color: this.colorValue,
          pageIndex: this.pageIndexValue,
        },
      });
    } else {
      this.delete();
    }
  }

  handleKeyDown(event) {
    if (event.key === 'Enter' && !event.shiftKey) {
      event.preventDefault();
      event.target.blur();
    }
  }

  delete(event) {
    if (event) event.stopPropagation();

    this.dispatch('deleted', {
      detail: { id: this.idValue },
    });

    this.element.remove();
  }

  showDeleteButton() {
    this.deleteButtonTarget.style.visibility = 'visible';
  }

  hideDeleteButton() {
    this.deleteButtonTarget.style.visibility = 'hidden';
  }

  handleMouseMove(event) {
    const rect = this.element.getBoundingClientRect();
    const isInHandleArea = event.clientX - rect.left < DRAG_HANDLE_SIZE;
    this.element.style.cursor = isInHandleArea ? 'move' : 'default';
  }

  startDragging(event) {
    const rect = this.element.getBoundingClientRect();
    const isInHandleArea = event.clientX - rect.left < DRAG_HANDLE_SIZE;

    if (isInHandleArea) {
      this.dragOffsetX = event.clientX - rect.left;
      this.dragOffsetY = event.clientY - rect.top;

      document.addEventListener('mousemove', (this.boundDragMove = this.dragMove.bind(this)));
      document.addEventListener('mouseup', (this.boundDragEnd = this.dragEnd.bind(this)));

      document.body.style.userSelect = 'none';
      event.preventDefault();
    }
  }

  dragMove(event) {
    const canvas = this.canvasElement;
    if (!canvas) return;

    const containerRect = canvas.getBoundingClientRect();
    const newX = event.clientX - containerRect.left - this.dragOffsetX;
    const newY = event.clientY - containerRect.top - this.dragOffsetY;

    this.element.style.left = `${newX}px`;
    this.element.style.top = `${newY}px`;
  }

  dragEnd() {
    this.dispatch('moved', {
      detail: {
        id: this.idValue,
        rect: this.getCurrentRect(),
      },
    });

    document.removeEventListener('mousemove', this.boundDragMove);
    document.removeEventListener('mouseup', this.boundDragEnd);
    document.body.style.userSelect = '';
  }

  getCurrentRect() {
    const canvas = this.canvasElement;
    if (!canvas) return [0, 0, 0, 0];

    const rect = this.element.getBoundingClientRect();
    const containerRect = canvas.getBoundingClientRect();
    const x = rect.left - containerRect.left;
    const y = rect.top - containerRect.top;

    return [x, y, x + rect.width, y + rect.height];
  }

  rgbToHex(rgb) {
    return `rgb(${rgb[0]}, ${rgb[1]}, ${rgb[2]})`;
  }
}
